"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const xstate_1 = require("xstate");
const interpreter_1 = require("xstate/lib/interpreter");
function useMachine(config, options, initialContext) {
    const machine = react_1.useMemo(() => xstate_1.Machine(config, options, initialContext), []);
    const [state, setStateSchema] = react_1.useState(machine.initialState);
    const [context, setContext] = react_1.useState(machine.context);
    const serviceRef = react_1.useRef(null);
    // Service is created lazily once
    function getService() {
        const service = serviceRef.current;
        if (service !== null) {
            return service;
        }
        const newService = interpreter_1.interpret(machine);
        // workaround https://github.com/DefinitelyTyped/DefinitelyTyped/issues/31065
        ;
        serviceRef.current = newService;
        newService.onTransition(state => setStateSchema(state));
        newService.onChange(context => setContext(context));
        // call init after the above callbacks are set so any immediate actions
        // are reflected in react's state
        newService.init();
        return newService;
    }
    // Stop the service when unmounting.
    react_1.useEffect(() => {
        return () => void getService().stop();
    }, []);
    return { state, send: getService().send, context, service: getService() };
}
exports.useMachine = useMachine;
