"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const TestRenderer = require("react-test-renderer");
const lodash_1 = require("lodash");
const _1 = require(".");
// simple ----------------------------------
var SimpleStateName;
(function (SimpleStateName) {
    SimpleStateName["GREEN"] = "GREEN";
    SimpleStateName["YELLOW"] = "YELLOW";
    SimpleStateName["RED"] = "RED";
})(SimpleStateName || (SimpleStateName = {}));
var SimpleAction;
(function (SimpleAction) {
    SimpleAction["TIMER"] = "TIMER";
})(SimpleAction || (SimpleAction = {}));
const SimpleInitialContext = {
    counter: 0
};
const simpleMachineConfig = {
    id: "light",
    initial: SimpleStateName.GREEN,
    states: {
        [SimpleStateName.GREEN]: {
            on: { TIMER: SimpleStateName.YELLOW }
        },
        [SimpleStateName.YELLOW]: {
            on: { TIMER: SimpleStateName.RED }
        },
        [SimpleStateName.RED]: {
            on: { TIMER: SimpleStateName.GREEN }
        }
    }
};
// side effect ----------------------------------
var SimpleStateNameWithSideEffect;
(function (SimpleStateNameWithSideEffect) {
    SimpleStateNameWithSideEffect["OFF"] = "OFF";
    SimpleStateNameWithSideEffect["ON"] = "ON";
})(SimpleStateNameWithSideEffect || (SimpleStateNameWithSideEffect = {}));
var SimpleActionWithSideEffect;
(function (SimpleActionWithSideEffect) {
    SimpleActionWithSideEffect["ACTIVATE"] = "ACTIVATE";
    SimpleActionWithSideEffect["DEACTIVATE"] = "DEACTIVATE";
})(SimpleActionWithSideEffect || (SimpleActionWithSideEffect = {}));
var SimpleCallbackActionWithSideEffect;
(function (SimpleCallbackActionWithSideEffect) {
    SimpleCallbackActionWithSideEffect["SWITCHED"] = "SWITCHED";
})(SimpleCallbackActionWithSideEffect || (SimpleCallbackActionWithSideEffect = {}));
const simpleInitialContextWithSideEffect = {
    counter: 0
};
const simpleMachineConfigWithSideEffect = {
    id: "switch",
    initial: SimpleStateNameWithSideEffect.OFF,
    states: {
        [SimpleStateNameWithSideEffect.OFF]: {
            on: {
                "": {
                    target: SimpleStateNameWithSideEffect.ON,
                    actions: [SimpleCallbackActionWithSideEffect.SWITCHED]
                }
            }
        },
        [SimpleStateNameWithSideEffect.ON]: {}
    }
};
describe("testing useMachine", () => {
    it("should successfully render a component with a simple machine and call useEffect", () => {
        const spy = jest.spyOn(React, "useEffect");
        const SimpleTestComponent = () => {
            const machine = _1.useMachine(simpleMachineConfig, {}, SimpleInitialContext);
            return React.createElement("div", null, machine.state.value);
        };
        let testComponent = TestRenderer.create(React.createElement(SimpleTestComponent, null));
        const renderedJSON = testComponent.toJSON();
        expect(lodash_1.get(renderedJSON, "children[0]")).toBe(SimpleStateName.GREEN);
        testComponent && testComponent.unmount();
        expect(spy).toHaveBeenCalled();
        spy.mockRestore();
    });
    it("should successfully re-render an updated component when send is called to change machine state", () => {
        const TestComponentWithAction = () => {
            const machine = _1.useMachine(simpleMachineConfig, {}, SimpleInitialContext);
            const sendTimer = () => {
                machine.send(SimpleAction.TIMER);
            };
            return (React.createElement(React.Fragment, null,
                React.createElement("div", null, machine.state.value),
                React.createElement("button", { onClick: sendTimer })));
        };
        const testComponent = TestRenderer.create(React.createElement(TestComponentWithAction, null));
        // check starting component state
        const initialRenderedJSON = testComponent.toJSON();
        expect(lodash_1.get(initialRenderedJSON, "[0].children[0]")).toBe(SimpleStateName.GREEN);
        // click button
        TestRenderer.act(() => testComponent.root &&
            testComponent.root.findByType("button").props.onClick());
        const afterClickRenderedJSON = testComponent.toJSON();
        // check component state after click
        expect(lodash_1.get(afterClickRenderedJSON, "[0].children[0]")).toBe(SimpleStateName.YELLOW);
    });
    it("should successfully execute a side-effect when one is included in the machine config", () => {
        const spy = jest.fn();
        const actionsMock = {
            switched: spy
        };
        const TestComponentWithSideEffect = () => {
            const machine = _1.useMachine(simpleMachineConfigWithSideEffect, {
                actions: {
                    [SimpleCallbackActionWithSideEffect.SWITCHED]: actionsMock.switched
                }
            }, simpleInitialContextWithSideEffect);
            return null;
        };
        TestRenderer.create(React.createElement(TestComponentWithSideEffect, null));
        expect(spy).toHaveBeenCalled();
        spy.mockRestore();
    });
});
